#    _____           ______  _____ 
#  / ____/    /\    |  ____ |  __ \
# | |        /  \   | |__   | |__) | Caer - Modern Computer Vision
# | |       / /\ \  |  __|  |  _  /  Languages: Python, C, C++, Cuda
# | |___   / ____ \ | |____ | | \ \  http://github.com/jasmcaus/caer
#  \_____\/_/    \_ \______ |_|  \_\

# Licensed under the MIT License <http://opensource.org/licenses/MIT>
# SPDX-License-Identifier: MIT
# Copyright (c) 2020-2021 The Caer Authors <http://github.com/jasmcaus>


__all__ = [
    'FPS',
    'FRAME_COUNT',
    'FRAME_WIDTH',
    'FRAME_HEIGHT'
]


## Video
FRAME_WIDTH = 3
FRAME_HEIGHT = 4
FPS = 5
FRAME_COUNT = 7  